'From Haver 5.0 [latest update: #4559] on 22 March 2021 at 6:11:49 pm'!
'Description An enhanced copy of Pharo''s write barrier code.'!
!provides: 'WriteBarrier' 1 21!
!requires: 'Cuis-Base' 50 3597 nil!
SystemOrganization addCategory: 'WriteBarrier'!





!classDefinition: #ModificationForbidden category: 'WriteBarrier'!
Error subclass: #ModificationForbidden
	instanceVariableNames: 'object fieldIndex newValue retrySelector'
	classVariableNames: 'ModificationForbiddenHandler'
	poolDictionaries: ''
	category: 'WriteBarrier'!
!classDefinition: 'ModificationForbidden class' category: 'WriteBarrier'!
ModificationForbidden class
	instanceVariableNames: ''!


!ModificationForbidden commentStamp: '<historical>' prior: 0!
This exception is raised when mutating a read-only object.

Optionally I support a class wide modification forbidden handler, 
preferably a block. This handler is sent a #value:value:value:value:
message and should handle the modification in what ever way it deems sensible.
If the block's evaluation answers true, the modification error is considered
handled and no exception is signaled. Otherwhise an exception is signaled.


My instances have 4 fields to be able to reproduce the modification through retryModification method.

object <Object> read-only object that the code attempted to mutate
index <SmallInteger> index of the field in the object mutated, relevant for the corresponding selector
value <Object> value that was attempted to be stored into the read-only object
retrySelector <Symbol> selector that can be used to reproduce the mutation (typically, #at:put:, #instVarAt:put:, etc.)
!

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
fieldIndex
	"Answer the value of fieldIndex"

	^ fieldIndex! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
fieldIndex: anObject
	"Set the value of fieldIndex"

	fieldIndex _ anObject! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
newValue
	"Answer the value of newValue"

	^ newValue! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
newValue: anObject
	"Set the value of newValue"

	newValue _ anObject! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
object
	"Answer the value of object"

	^ object! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
object: anObject
	"Set the value of object"

	object _ anObject! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
retrySelector
	"Answer the value of retrySelector"

	^ retrySelector! !

!ModificationForbidden methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:05:43'!
retrySelector: anObject
	"Set the value of retrySelector"

	retrySelector _ anObject! !

!ModificationForbidden methodsFor: 'handling' stamp: 'klg 11/2/2018 08:06:53'!
retryModification
	object perform: retrySelector with: fieldIndex with: newValue.
	self resume: newValue! !

!ModificationForbidden methodsFor: 'private' stamp: 'klg 11/2/2018 08:55:06'!
isResumable
	"Determine whether an exception is resumable."

	^true! !

!ModificationForbidden class methodsFor: 'exceptionInstantiator' stamp: 'KLG 6/20/2019 15:01:55'!
signalFor: anObject atIndex: fieldIndex with: newValue selector: aSelector
	"Signal a write barrier exception resulting from an #at:put: send."

	ModificationForbiddenHandler 
		value: anObject 
		value: fieldIndex 
		value: newValue 
		value: aSelector :: ifFalse: [
			self new 
				object: anObject;
				fieldIndex: fieldIndex;
				newValue: newValue;
				retrySelector: aSelector :: signal ]! !

!ModificationForbidden class methodsFor: 'exceptionInstantiator' stamp: 'KLG 6/20/2019 15:06:10'!
signalFor: anObject atInstVar: fieldIndex with: newValue
	"Signal a write barrier exception resulting from an instance variable change."

	self 
		signalFor: anObject
		atIndex: fieldIndex
		with: newValue
		selector:  #instVarAt:put:! !

!ModificationForbidden class methodsFor: 'global handling' stamp: 'KLG 6/20/2019 15:50:13'!
handleModificationForbidden: aHandlingObject with: aSelector
	"Install a modification forbidden handler.
	
	Answer the old handler."

	| answer |
	answer _ ModificationForbiddenHandler.
	ModificationForbiddenHandler _ [ :object :index :value :retrySelector |
		aHandlingObject perform: aSelector withArguments: {object. index. value. retrySelector} ].
	^ answer ! !

!ModificationForbidden class methodsFor: 'global handling' stamp: 'KLG 6/20/2019 15:46:25'!
modificationForbiddenHandler: aHandler
	"Install a modification forbidden handler.
	
	Answer the old handler."

	| answer |
	answer _ ModificationForbiddenHandler.
	ModificationForbiddenHandler _ aHandler.
	^ answer ! !

!ModificationForbidden class methodsFor: 'global handling' stamp: 'KLG 6/20/2019 15:24:45'!
resetModificationForbiddenHandler
	"Reset the modification forbidden handler to the all ignoring default state.
	
	Answer the old handler."

	| answer |
	answer _ ModificationForbiddenHandler.
	ModificationForbiddenHandler _ [ :object :index :value :retrySelector | false ].
	^ answer! !

!ModificationForbidden class methodsFor: 'class initialization' stamp: 'KLG 6/20/2019 15:23:54'!
initialize
	"Initialize the modification forbidden handler."

	super initialize.
	self resetModificationForbiddenHandler! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/6/2019 22:20:54'!
at: index put: value 
	"Primitive. Assumes receiver is indexable. Store the argument value in 
	the indexable element of the receiver indicated by index. Fail if the 
	index is not an Integer or is out of bounds. Or fail if the value is not of 
	the right type for this kind of collection. Answer the value that was 
	stored. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 61 error: ec>
	index isInteger ifTrue:
		[self class isVariable
			ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [
						self isReadOnlyObject and: [ ec == #'no modification' ] ::
							ifTrue: [
								^ self modificationForbiddenClass
									signalFor: self
									atIndex: index 
									with: value 
									selector:  #at:put: ].
						self errorImproperStore ]
					ifFalse: [self errorSubscriptBounds: index]]
			ifFalse: [self errorNotIndexable]].
	index isNumber
		ifTrue: [^self at: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/5/2019 20:19:57'!
attemptToAssign: value withIndex: index 
	"Called by the VM when assigning an instance variable of an immutable object.
	Upon return, executing will resume *after* the inst var assignment. If the inst var mutation has to be 
	performed, do it manually here in the call back with instVarAt:put:.
	This method has to return *no* value by jumping to the context's sender"
	
	self modificationForbiddenClass signalFor: self atInstVar: index with: value.

	thisContext sender jump
	"CAN'T REACH"! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/6/2019 22:16:07'!
basicAt: index put: value 
	"Primitive. Assumes receiver is indexable. Store the second argument 
	value in the indexable element of the receiver indicated by index. Fail 
	if the index is not an Integer or is out of bounds. Or fail if the value is 
	not of the right type for this kind of collection. Answer the value that 
	was stored. Essential. Do not override in a subclass. See Object 
	documentation whatIsAPrimitive."

	<primitive: 61 error: ec>
	index isInteger
		ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [
						self isReadOnlyObject and: [ ec == #'no modification' ] ::
							ifTrue: [
								^ self modificationForbiddenClass 
									signalFor: self 
									atIndex: index 
									with: value 
									selector: #basicAt:put: ].
						self errorImproperStore]
					ifFalse: [self errorSubscriptBounds: index]].
	index isNumber
		ifTrue: [^self basicAt: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*WriteBarrier' stamp: 'klg 11/2/2018 07:50:44'!
beReadOnlyObject
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 Set the read-only flag of the receiver to true and answer the previous vaue of the flag."
	^self setIsReadOnlyObject: true! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/3/2019 23:05:44'!
beReadOnlyObjectIfSensible
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 Set the read-only flag of the receiver to true and answer the previous vaue of the flag.
	
	Only perform the operation if it makes sense."
	^ self setIsReadOnlyObjectIfSensible: true! !

!Object methodsFor: '*WriteBarrier' stamp: 'klg 11/2/2018 07:50:15'!
beWritableObject
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 Set the read-only flag of the receiver to false and answer the previous vaue of the flag."
	^self setIsReadOnlyObject: false! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/3/2019 23:06:22'!
beWritableObjectIfSensible
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 Set the read-only flag of the receiver to false and answer the previous vaue of the flag.
	
	Only perform the operation if it makes sense."
	^self setIsReadOnlyObjectIfSensible: false! !

!Object methodsFor: '*WriteBarrier' stamp: 'klg 11/2/2018 07:46:23'!
isReadOnlyObject
	"Answer if the receiver is read-only.
	 If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code."
	<primitive: 163 error: ec>
	^self primitiveFailed! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/5/2019 21:10:24'!
modificationForbiddenClass
	"Answer the ModificationForbidden excpetion class to use."

	^ self class modificationForbiddenClass! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/6/2019 22:17:19'!
rawBasicAt: index put: value 
	"Primitive. Assumes receiver is indexable. Store the second argument 
	value in the indexable element of the receiver indicated by index. Fail 
	if the index is not an Integer or is out of bounds. Or fail if the value is 
	not of the right type for this kind of collection. Answer the value that 
	was stored. Essential. Do not override in a subclass. See Object 
	documentation whatIsAPrimitive."

	<primitive: 61 error: ec>
	index isInteger
		ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [
						self isReadOnlyObject and: [ ec == #'no modification' ] ::
							ifTrue: [
								^ self modificationForbiddenClass 
									signalFor: self 
									atIndex: index 
									with: value 
									selector: #rawBasicAt:put: ].
						self errorImproperStore]
					ifFalse: [self errorSubscriptBounds: index]].
	index isNumber
		ifTrue: [^self basicAt: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/3/2019 22:34:54'!
setIsReadOnlyObject: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)"
	<primitive: 164 error: ec>
	ec == #'bad receiver' ifTrue:
		[ ^ self error: 'object does not support write-barriers' ].
	ec == #'inappropriate operation' ifTrue:
	     [ ^ self error: 'object does not suport read-only instance variables' ].
	ec == #'bad argument' ifTrue:
		[ ^ self error: 'argument must be boolean' ].
	^ self primitiveFailed! !

!Object methodsFor: '*WriteBarrier' stamp: 'KLG 2/4/2019 20:24:40'!
setIsReadOnlyObjectIfSensible: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)
	
	This primitive is meant to be redefined in subclasses that are unboxed,
	such as instances of SmallInteger, and therefore truly immutable."
	^ self setIsReadOnlyObject: aBoolean! !

!Object class methodsFor: '*writebarrier' stamp: 'KLG 6/20/2019 19:25:03'!
modificationForbiddenClass
	"Answer the ModificationForbidden excpetion class to use."

	^ ModificationForbidden! !

!SmallFloat64 methodsFor: '*WriteBarrier' stamp: 'KLG 2/4/2019 20:25:42'!
setIsReadOnlyObjectIfSensible: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)
	
	This primitive is meant to be redefined in subclasses that are unboxed,
	such as instances of SmallInteger, and therefore truly immutable.
	
	Do nothing because I am immutable.
	Answer true, because the previous state was also immutable."
	^ true! !

!SmallInteger methodsFor: '*writebarrier' stamp: 'KLG 2/3/2019 22:45:13'!
setIsReadOnlyObjectIfSensible: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)
	
	This primitive is meant to be redefined in subclasses that are unboxed,
	such as instances of SmallInteger, and therefore truly immutable.
	
	Do nothing because I am immutable.
	Answer true, because the previous state was also immutable."
	^ true! !

!String methodsFor: '*WriteBarrier-accessing' stamp: 'KLG 1/3/2021 22:16:58'!
at: index put: aCharacter 
	"Primitive. Store the Character in the field of the receiver indicated by
	the index. Fail if the index is not an Integer or is out of bounds, or if
	the argument is not a Character. Essential. See Object documentation
	whatIsAPrimitive."

	<primitive: 64 error: ec>
	(aCharacter is: #Character)
		ifTrue: [	
			self isReadOnlyObject and: [ ec == #'no modification' ] ::
				ifTrue: [
					^ self modificationForbiddenClass
						signalFor: self
						atIndex: index 
						with: aCharacter
						selector:  #at:put: ].
			index isInteger
				ifTrue: [ self errorSubscriptBounds: index ]
				ifFalse: [ self errorNonIntegerIndex ]]
		ifFalse: [ self error: 'Strings only store Characters' ]! !

!OrderedCollection methodsFor: '*WriteBarrier' stamp: 'KLG 1/3/2021 22:06:22'!
setIsReadOnlyObject: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)"
	array setIsReadOnlyObject: aBoolean.
	^ super setIsReadOnlyObject: aBoolean! !

!Set methodsFor: '*WriteBarrier' stamp: 'KLG 1/3/2021 22:05:25'!
setIsReadOnlyObject: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)"
	array setIsReadOnlyObject: aBoolean.
	^ super setIsReadOnlyObject: aBoolean! !

!Array2D methodsFor: '*WriteBarrier' stamp: 'KLG 1/3/2021 22:04:25'!
setIsReadOnlyObject: aBoolean
	"If the VM supports read-only objects it will not write to read-only objects.
	 An attempt to write to an instance variable of a read-only object will
	 cause the VM to send attemptToAssign:withIndex: to the read-only object.
	 An attempt to modify a read-only object in a primitive will cause the
	 primitive to fail with a #'no modification' error code.
	 This primitive sets the read-only flag of the receiver to the given
	 value and answers the previous vaue of the flag.
	 Note: Some objects can't be read-only, currently contexts and objects related
	 to process scheduling (Processor, Process instances, Semaphore instances, ...)"
	elements setIsReadOnlyObject: aBoolean.
	^ super setIsReadOnlyObject: aBoolean! !
ModificationForbidden initialize!
