'From Haver 5.0 [latest update: #4559] on 22 March 2021 at 6:17:30 pm'!
'Description Tests for the write barrier package.'!
!provides: 'WriteBarrierTest' 1 9!
!requires: 'WriteBarrier' 1 19 nil!
SystemOrganization addCategory: 'WriteBarrierTest'!


!moduleCreation: WriteBarrierTest!
Modules newEnvironment: #WriteBarrierTest!

!interfacesOf: WriteBarrierTest!
Modules environment: #WriteBarrierTest ::
	interface: #API exporting: #() ::
	interface: #SPI aliasFor: #API!


!classDefinition: (Modules>>#WriteBarrierTest>>#WriteBarrierTest) category: 'WriteBarrierTest'!
TestCase subclass: #WriteBarrierTest
	instanceVariableNames: 'badReceiversToTest'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrierTest'
	inModule: #WriteBarrierTest!
!classDefinition: (Modules>>#WriteBarrierTest>>#WriteBarrierTest) class category: 'WriteBarrierTest'!
(Modules>>#WriteBarrierTest>>#WriteBarrierTest) class
	instanceVariableNames: ''!

!classDefinition: (Modules>>#WriteBarrierTest>>#WriteBarrierTester) category: 'WriteBarrierTest'!
Object subclass: #WriteBarrierTester
	instanceVariableNames: 'iv0 iv1 didChange'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'WriteBarrierTest'
	inModule: #WriteBarrierTest!
!classDefinition: (Modules>>#WriteBarrierTest>>#WriteBarrierTester) class category: 'WriteBarrierTest'!
(Modules>>#WriteBarrierTest>>#WriteBarrierTester) class
	instanceVariableNames: ''!


!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) commentStamp: '<historical>' prior: 0!
A test for the write barrier code.!

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) commentStamp: '<historical>' prior: 0!
I am a tester for write barriers.!

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Exception Handling' stamp: 'KLG 6/20/2019 15:40:30'!
handleModificationForbiddenFor: anObject atIndex: fieldIndex with: newValue selector: aSelector
	"An exception handling method to test global messages."

	self 
		assert: aSelector == #instVarAt:put:;
		assert: 1 equals: newValue;
		assert: 2 equals: fieldIndex.
	^ true! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 2/3/2019 22:55:50'!
testBadArgument
	"Test that that a bad receiver error is signaled."

	self 
		should: [ '42' setIsReadOnlyObject: nil] 
		raise: Error withExceptionDo: [ :exception |
			exception messageText = 'argument must be boolean' ]
! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 2/4/2019 20:07:28'!
testBadReceiver
	"Test that that a bad receiver error is signaled."

	badReceiversToTest do: [ :badReceiver |
		self 
			should: [ badReceiver beReadOnlyObject ] 
			raise: Error withExceptionDo: [ :exception |
				exception messageText = 'object does not support write-barriers' ];
			should: [ badReceiver beWritableObject ] 
			raise: Error withExceptionDo: [ :exception |
				exception messageText = 'object does not support write-barriers' ] ]! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 2/5/2019 20:14:42'!
testExceptionHandlingForArrays
	"Test the exception handling for write barrieres."

	| tester |
	tester _ {1. 2. 3}.
	self assert: (tester at: 1) equals: 1.
	tester beReadOnlyObject.
	self 
		should: [ tester at: 1 put: #no ] 
		raise: ModificationForbidden 
		withExceptionDo: [ :exception |
			self assert: exception retrySelector == #at:put: ].
	self assert: (tester at: 2) equals: 2.
	self assert: (tester at: 1) equals: 1.
	tester beWritableObject.
	tester at: 2 put: #two.
	self assert: #two equals: (tester at: 2)! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 2/4/2019 21:15:11'!
testExceptionHandlingForInstanceVariables
	"Test the exception handling for write barrieres."

	| tester |
	tester _ WriteBarrierTester new.
	tester iv0: 0.
	self assert: tester iv0 equals: 0.
	tester beReadOnlyObject.
	self should: [ tester iv1: 1 ] raise: ModificationForbidden withExceptionDo: [ :exception |
		self assert: exception retrySelector == #instVarAt:put: ].
	self assert: tester iv1 equals: nil.
	self assert: tester iv0 equals: 0.
	tester beWritableObject.
	tester iv1: 1.
	self assert: tester iv1 equals: 1.
! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'klg 11/2/2018 09:05:35'!
testExceptionResuming
	"Test the exception resuming for write barrieres."

	| tester |
	tester _ WriteBarrierTester new.
	self deny: tester didChange.
	self assert: tester iv0 equals: nil.
	tester beReadOnlyObject.
	self assert: (tester handledIv0: #zero) equals: #zero.
	self assert: tester didChange.
	self assert: tester iv0 equals: #zero.
	self assert: (tester handledIv0: #meMySelfAndI) equals: #meMySelfAndI.
	self assert: tester iv0 equals: #meMySelfAndI.
	! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 6/20/2019 15:32:08'!
testGlobalModificationHandlingForInstanceVariables


	| tester |
	tester _ WriteBarrierTester new.
	tester iv0: 0.
	self assert: tester iv0 equals: 0.
	tester beReadOnlyObject.
	ModificationForbidden modificationForbiddenHandler: 
		[ :object :index :value : retrySelector |
			self 
				assert: retrySelector == #instVarAt:put:;
				assert: object == tester;
				assert: 1 equals: value;
				assert: 2 equals: index.
			true ].
	tester iv1: 1.
	ModificationForbidden resetModificationForbiddenHandler.
	self assert: tester iv1 equals: nil.
	self assert: tester iv0 equals: 0.
	tester beWritableObject.
	tester iv1: 1.
	self assert: tester iv1 equals: 1.
! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 6/20/2019 15:47:55'!
testGlobalModificationHandlingWithAMessageForInstanceVariables


	| tester |
	tester _ WriteBarrierTester new.
	tester iv0: 0.
	self assert: tester iv0 equals: 0.
	tester beReadOnlyObject.
	ModificationForbidden handleModificationForbidden: self with: 
		#handleModificationForbiddenFor:atIndex:with:selector:.
	tester iv1: 1.
	ModificationForbidden resetModificationForbiddenHandler.
	self assert: tester iv1 equals: nil.
	self assert: tester iv0 equals: 0.
	tester beWritableObject.
	tester iv1: 1.
	self assert: tester iv1 equals: 1.
! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 2/3/2019 23:00:15'!
testNoReadonlySupport
	"Test that that a bad receiver error is signaled."

	self 
		should: [ thisContext beReadOnlyObject ] 
		raise: Error withExceptionDo: [ :exception |
			exception messageText = 'object does not suport read-only instance variables' ].
	thisContext beWritableObject ":: This however should work"! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'KLG 2/4/2019 20:23:13'!
testSensibleWriteBarrierSupport
	"Test the writebarrier support where it makes sense."

	badReceiversToTest do: [ :badReceiver |
		self assert: badReceiver isReadOnlyObject.
		badReceiver beReadOnlyObjectIfSensible.
		self assert: badReceiver isReadOnlyObject.
		badReceiver beWritableObjectIfSensible.
		self assert: badReceiver isReadOnlyObject ]
! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Testing' stamp: 'klg 11/2/2018 08:26:35'!
testSetIsReadOnlyObject
	"Test the read only flag in this image"

	| to |
	to _ Object new.
	to beReadOnlyObject.
	self assert: to isReadOnlyObject.
     to beWritableObject.
	self deny: to isReadOnlyObject.
! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Running' stamp: 'KLG 6/20/2019 15:51:33'!
setUp
	"Set up the bad receivers to test."
	
	super setUp.
	badReceiversToTest _ OrderedCollection with: 42 .
	42.0 isKindOf: SmallFloat64 :: ifTrue: [
		badReceiversToTest add: 42.0 ]
	! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTest) methodsFor: 'Running' stamp: 'KLG 6/20/2019 15:51:19'!
tearDown
	"Set up the bad receivers to test."
	
	super tearDown.
	ModificationForbidden resetModificationForbiddenHandler
	! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:41:03'!
didChange
	"Answer the value of didChange"

	self assert: didChange notNil.
	^ didChange! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:43:03'!
didChange: anObject
	"Set the value of didChange"

	didChange _ anObject! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 09:03:50'!
handledIv0: anObject
	"Set the value of iv0"

	^[ iv0 _ anObject ] on: ModificationForbidden do: 
		[ :exception |
			self beWritableObject.
			self iv0: anObject.
			didChange _ true.
			exception resume ].
		! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:20:29'!
iv0
	"Answer the value of iv0"

	^ iv0! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:20:29'!
iv0: anObject
	"Set the value of iv0"

	iv0 _ anObject! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:20:29'!
iv1
	"Answer the value of iv1"

	^ iv1! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'accessing' stamp: 'klg 11/2/2018 08:20:29'!
iv1: anObject
	"Set the value of iv1"

	iv1 _ anObject! !

!(Modules>>#WriteBarrierTest>>#WriteBarrierTester) methodsFor: 'initialization' stamp: 'klg 11/2/2018 08:54:03'!
initialize
	super initialize.
	didChange _ false! !
